@@ -1,8 +1,8 @@
 /* -*- C -*-
- * $Id: curses.c 12095 2007-03-19 03:58:57Z matz $
+ * $Id: curses.c 25286 2009-10-10 10:49:47Z akr $
  *
  * ext/curses/curses.c
- * 
+ *
  * by MAEDA Shugo (ender@pic-internet.or.jp)
  * modified by Yukihiro Matsumoto (matz@netlab.co.jp),
  *         Toki Yoshinori,
@@ -14,7 +14,7 @@
  */
 
 #include "ruby.h"
-#include "rubyio.h"
+#include "ruby/io.h"
 
 #if defined(HAVE_NCURSES_H)
 # include <ncurses.h>
@@ -94,7 +94,7 @@ free_window(struct windata *winp)
 {
     if (winp->window && winp->window != stdscr) delwin(winp->window);
     winp->window = 0;
-    free(winp);
+    xfree(winp);
 }
 
 static VALUE
@@ -110,8 +110,8 @@ prep_window(VALUE class, WINDOW *window)
     obj = rb_obj_alloc(class);
     Data_Get_Struct(obj, struct windata, winp);
     winp->window = window;
-    
-    return obj;    
+
+    return obj;
 }
 
 /*-------------------------- module Curses --------------------------*/
@@ -138,6 +138,7 @@ curses_init_screen(void)
 static VALUE
 curses_close_screen(void)
 {
+    curses_stdscr();
 #ifdef HAVE_ISENDWIN
     if (!isendwin())
 #endif
@@ -159,19 +160,20 @@ curses_finalize(VALUE dummy)
     rb_gc_unregister_address(&rb_stdscr);
 }
 
+#ifdef HAVE_ISENDWIN
 /* def closed? */
 static VALUE
 curses_closed(void)
 {
-#ifdef HAVE_ISENDWIN
+    curses_stdscr();
     if (isendwin()) {
 	return Qtrue;
     }
     return Qfalse;
-#else
-    rb_notimplement();
-#endif
 }
+#else
+#define curses_closed rb_f_notimplement
+#endif
 
 /* def clear */
 static VALUE
@@ -307,18 +309,40 @@ curses_flash(VALUE obj)
     return Qnil;
 }
 
+static int
+curses_char(VALUE c)
+{
+    if (FIXNUM_P(c)) {
+	return NUM2INT(c);
+    }
+    else {
+	int cc;
+
+	StringValue(c);
+	if (RSTRING_LEN(c) == 0 || RSTRING_LEN(c) > 1) {
+	    rb_raise(rb_eArgError, "string not corresponding a character");
+	}
+	cc = RSTRING_PTR(c)[0];
+	if (cc > 0x7f) {
+	    rb_raise(rb_eArgError, "no multibyte string supported (yet)");
+	}
+	return cc;
+    }
+}
+
+#ifdef HAVE_UNGETCH
 /* def ungetch */
 static VALUE
 curses_ungetch(VALUE obj, VALUE ch)
 {
-#ifdef HAVE_UNGETCH
+    int c = curses_char(ch);
     curses_stdscr();
-    ungetch(NUM2INT(ch));
-#else
-    rb_notimplement();
-#endif
+    ungetch(c);
     return Qnil;
 }
+#else
+#define curses_ungetch rb_f_notimplement
+#endif
 
 /* def setpos(y, x) */
 static VALUE
@@ -333,6 +357,7 @@ curses_setpos(VALUE obj, VALUE y, VALUE x)
 static VALUE
 curses_standout(VALUE obj)
 {
+    curses_stdscr();
     standout();
     return Qnil;
 }
@@ -341,6 +366,7 @@ curses_standout(VALUE obj)
 static VALUE
 curses_standend(VALUE obj)
 {
+    curses_stdscr();
     standend();
     return Qnil;
 }
@@ -375,41 +401,71 @@ curses_insch(VALUE obj, VALUE ch)
 static VALUE
 curses_addstr(VALUE obj, VALUE str)
 {
+    StringValue(str);
+    str = rb_str_export_locale(str);
     curses_stdscr();
     if (!NIL_P(str)) {
-	addstr(STR2CSTR(str));
+	addstr(StringValueCStr(str));
     }
     return Qnil;
 }
 
+static VALUE
+getch_func(void *arg)
+{
+    int *ip = (int *)arg;
+    *ip = getch();
+    return Qnil;
+}
+
 /* def getch */
 static VALUE
 curses_getch(VALUE obj)
 {
-    rb_read_check(stdin);
+    int c;
+
     curses_stdscr();
-    return UINT2NUM(getch());
+    rb_thread_blocking_region(getch_func, (void *)&c, RUBY_UBF_IO, 0);
+    if (c == EOF) return Qnil;
+    if (rb_isprint(c)) {
+	char ch = (char)c;
+
+	return rb_locale_str_new(&ch, 1);
+    }
+    return UINT2NUM(c);
+}
+
+/* This should be big enough.. I hope */
+#define GETSTR_BUF_SIZE 1024
+
+static VALUE
+getstr_func(void *arg)
+{
+    char *rtn = (char *)arg;
+#if defined(HAVE_GETNSTR)
+    getnstr(rtn,GETSTR_BUF_SIZE-1);
+#else
+    getstr(rtn);
+#endif
+    return Qnil;
 }
 
 /* def getstr */
 static VALUE
 curses_getstr(VALUE obj)
 {
-    char rtn[1024]; /* This should be big enough.. I hope */
+    char rtn[GETSTR_BUF_SIZE];
 
-    rb_read_check(stdin);
-#if defined(HAVE_GETNSTR)
-    getnstr(rtn,1023);
-#else
-    getstr(rtn);
-#endif
-    return rb_tainted_str_new2(rtn);
+    curses_stdscr();
+    rb_thread_blocking_region(getstr_func, (void *)rtn, RUBY_UBF_IO, 0);
+    return rb_locale_str_new_cstr(rtn);
 }
 
 /* def delch */
 static VALUE
 curses_delch(VALUE obj)
 {
+    curses_stdscr();
     delch();
     return Qnil;
 }
@@ -418,6 +474,7 @@ curses_delch(VALUE obj)
 static VALUE
 curses_deleteln(VALUE obj)
 {
+    curses_stdscr();
 #if defined(HAVE_DELETELN) || defined(deleteln)
     deleteln();
 #endif
@@ -428,6 +485,7 @@ curses_deleteln(VALUE obj)
 static VALUE
 curses_insertln(VALUE obj)
 {
+    curses_stdscr();
 #if defined(HAVE_INSERTLN) || defined(insertln)
     insertln();
 #endif
@@ -439,16 +497,19 @@ static VALUE
 curses_keyname(VALUE obj, VALUE c)
 {
 #ifdef HAVE_KEYNAME
-  const char *name;
+    int cc = curses_char(c);
+    const char *name;
 
-  name = keyname(NUM2INT(c));
-  if (name) {
-    return rb_str_new2(name);
-  } else {
-    return Qnil;
-  }
+    curses_stdscr();
+    name = keyname(cc);
+    if (name) {
+	return rb_str_new_cstr(name);
+    }
+    else {
+	return Qnil;
+    }
 #else
-  return Qnil;
+    return Qnil;
 #endif
 }
 
@@ -464,86 +525,155 @@ curses_cols(void)
     return INT2FIX(COLS);
 }
 
+/**
+ * Sets Cursor Visibility.
+ * 0: invisible
+ * 1: visible
+ * 2: very visible
+ */
 static VALUE
 curses_curs_set(VALUE obj, VALUE visibility)
 {
 #ifdef HAVE_CURS_SET
-  int n;
-  return (n = curs_set(NUM2INT(visibility)) != ERR) ? INT2FIX(n) : Qnil;
+    int n;
+    curses_stdscr();
+    return (n = curs_set(NUM2INT(visibility)) != ERR) ? INT2FIX(n) : Qnil;
 #else
-  return Qnil;
+    return Qnil;
 #endif
 }
 
 static VALUE
 curses_scrl(VALUE obj, VALUE n)
 {
-  /* may have to raise exception on ERR */
+    /* may have to raise exception on ERR */
 #ifdef HAVE_SCRL
-  return (scrl(NUM2INT(n)) == OK) ? Qtrue : Qfalse;
+    curses_stdscr();
+    return (scrl(NUM2INT(n)) == OK) ? Qtrue : Qfalse;
 #else
-  return Qfalse;
+    return Qfalse;
 #endif
 }
 
 static VALUE
 curses_setscrreg(VALUE obj, VALUE top, VALUE bottom)
 {
-  /* may have to raise exception on ERR */
+    /* may have to raise exception on ERR */
 #ifdef HAVE_SETSCRREG
-  return (setscrreg(NUM2INT(top), NUM2INT(bottom)) == OK) ? Qtrue : Qfalse;
+    curses_stdscr();
+    return (setscrreg(NUM2INT(top), NUM2INT(bottom)) == OK) ? Qtrue : Qfalse;
 #else
-  return Qfalse;
+    return Qfalse;
 #endif
 }
 
 static VALUE
 curses_attroff(VALUE obj, VALUE attrs)
 {
-  return window_attroff(rb_stdscr,attrs);  
-  /* return INT2FIX(attroff(NUM2INT(attrs))); */
+    curses_stdscr();
+    return window_attroff(rb_stdscr,attrs);
+    /* return INT2FIX(attroff(NUM2INT(attrs))); */
 }
 
 static VALUE
 curses_attron(VALUE obj, VALUE attrs)
 {
-  return window_attron(rb_stdscr,attrs);
-  /* return INT2FIX(attroff(NUM2INT(attrs))); */
+    curses_stdscr();
+    return window_attron(rb_stdscr,attrs);
+    /* return INT2FIX(attroff(NUM2INT(attrs))); */
 }
 
 static VALUE
 curses_attrset(VALUE obj, VALUE attrs)
 {
-  return window_attrset(rb_stdscr,attrs);
-  /* return INT2FIX(attroff(NUM2INT(attrs))); */
+    curses_stdscr();
+    return window_attrset(rb_stdscr,attrs);
+    /* return INT2FIX(attroff(NUM2INT(attrs))); */
 }
 
 static VALUE
 curses_bkgdset(VALUE obj, VALUE ch)
 {
 #ifdef HAVE_BKGDSET
-  bkgdset(NUM2CH(ch));
+    curses_stdscr();
+    bkgdset(NUM2CH(ch));
 #endif
-  return Qnil;
+    return Qnil;
 }
 
 static VALUE
 curses_bkgd(VALUE obj, VALUE ch)
 {
 #ifdef HAVE_BKGD
-  return (bkgd(NUM2CH(ch)) == OK) ? Qtrue : Qfalse;
+    curses_stdscr();
+    return (bkgd(NUM2CH(ch)) == OK) ? Qtrue : Qfalse;
 #else
-  return Qfalse;
+    return Qfalse;
 #endif
 }
 
+#if defined(HAVE_USE_DEFAULT_COLORS)
+static VALUE
+curses_use_default_colors(VALUE obj)
+{
+    curses_stdscr();
+    use_default_colors();
+    return Qnil;
+}
+#else
+#define curses_use_default_colors rb_f_notimplement
+#endif
+
+#if defined(HAVE_TABSIZE)
+static VALUE
+curses_tabsize_set(VALUE obj, VALUE val)
+{
+    TABSIZE = NUM2INT(val);
+    return INT2NUM(TABSIZE);
+}
+#else
+#define curses_tabsize_set rb_f_notimplement
+#endif
+
+#if defined(HAVE_TABSIZE)
+static VALUE
+curses_tabsize_get(VALUE ojb)
+{
+    return INT2NUM(TABSIZE);
+}
+#else
+#define curses_tabsize_get rb_f_notimplement
+#endif
+
+#if defined(HAVE_ESCDELAY)
+static VALUE
+curses_escdelay_set(VALUE obj, VALUE val)
+{
+    ESCDELAY = NUM2INT(val);
+    return INT2NUM(ESCDELAY);
+}
+#else
+#define curses_escdelay_set rb_f_notimplement
+#endif
+
+#if defined(HAVE_ESCDELAY)
+static VALUE
+curses_escdelay_get(VALUE obj)
+{
+    return INT2NUM(ESCDELAY);
+}
+#else
+#define curses_escdelay_get rb_f_notimplement
+#endif
+
 static VALUE
 curses_resizeterm(VALUE obj, VALUE lin, VALUE col)
 {
 #if defined(HAVE_RESIZETERM)
-  return (resizeterm(NUM2INT(lin),NUM2INT(col)) == OK) ? Qtrue : Qfalse;
+    curses_stdscr();
+    return (resizeterm(NUM2INT(lin),NUM2INT(col)) == OK) ? Qtrue : Qfalse;
 #else
-  return Qnil;
+    return Qnil;
 #endif
 }
 
@@ -551,77 +681,106 @@ curses_resizeterm(VALUE obj, VALUE lin, VALUE col)
 static VALUE
 curses_start_color(VALUE obj)
 {
-  /* may have to raise exception on ERR */
-  return (start_color() == OK) ? Qtrue : Qfalse;
+    /* may have to raise exception on ERR */
+    curses_stdscr();
+    return (start_color() == OK) ? Qtrue : Qfalse;
 }
 
 static VALUE
 curses_init_pair(VALUE obj, VALUE pair, VALUE f, VALUE b)
 {
-  /* may have to raise exception on ERR */
-  return (init_pair(NUM2INT(pair),NUM2INT(f),NUM2INT(b)) == OK) ? Qtrue : Qfalse;
+    /* may have to raise exception on ERR */
+    curses_stdscr();
+    return (init_pair(NUM2INT(pair),NUM2INT(f),NUM2INT(b)) == OK) ? Qtrue : Qfalse;
 }
 
 static VALUE
 curses_init_color(VALUE obj, VALUE color, VALUE r, VALUE g, VALUE b)
 {
-  /* may have to raise exception on ERR */
-  return (init_color(NUM2INT(color),NUM2INT(r),
-		     NUM2INT(g),NUM2INT(b)) == OK) ? Qtrue : Qfalse;
+    /* may have to raise exception on ERR */
+    curses_stdscr();
+    return (init_color(NUM2INT(color),NUM2INT(r),
+		       NUM2INT(g),NUM2INT(b)) == OK) ? Qtrue : Qfalse;
 }
 
 static VALUE
 curses_has_colors(VALUE obj)
 {
-  return has_colors() ? Qtrue : Qfalse;
+    curses_stdscr();
+    return has_colors() ? Qtrue : Qfalse;
 }
 
 static VALUE
 curses_can_change_color(VALUE obj)
 {
-  return can_change_color() ? Qtrue : Qfalse;
+    curses_stdscr();
+    return can_change_color() ? Qtrue : Qfalse;
 }
 
+#if defined(HAVE_COLORS)
+static VALUE
+curses_colors(VALUE obj)
+{
+    return INT2FIX(COLORS);
+}
+#else
+#define curses_colors rb_f_notimplement
+#endif
+
 static VALUE
 curses_color_content(VALUE obj, VALUE color)
 {
-  short r,g,b;
+    short r,g,b;
 
-  color_content(NUM2INT(color),&r,&g,&b);
-  return rb_ary_new3(3,INT2FIX(r),INT2FIX(g),INT2FIX(b));
+    curses_stdscr();
+    color_content(NUM2INT(color),&r,&g,&b);
+    return rb_ary_new3(3,INT2FIX(r),INT2FIX(g),INT2FIX(b));
 }
 
+
+#if defined(HAVE_COLOR_PAIRS)
+static VALUE
+curses_color_pairs(VALUE obj)
+{
+    return INT2FIX(COLOR_PAIRS);
+}
+#else
+#define curses_color_pairs rb_f_notimplement
+#endif
+
 static VALUE
 curses_pair_content(VALUE obj, VALUE pair)
 {
-  short f,b;
+    short f,b;
 
-  pair_content(NUM2INT(pair),&f,&b);
-  return rb_ary_new3(2,INT2FIX(f),INT2FIX(b));
+    curses_stdscr();
+    pair_content(NUM2INT(pair),&f,&b);
+    return rb_ary_new3(2,INT2FIX(f),INT2FIX(b));
 }
 
 static VALUE
 curses_color_pair(VALUE obj, VALUE attrs)
 {
-  return INT2FIX(COLOR_PAIR(NUM2INT(attrs)));
+    return INT2FIX(COLOR_PAIR(NUM2INT(attrs)));
 }
 
 static VALUE
 curses_pair_number(VALUE obj, VALUE attrs)
 {
-  return INT2FIX(PAIR_NUMBER(NUM2INT(attrs)));
+    curses_stdscr();
+    return INT2FIX(PAIR_NUMBER(NUM2INT(attrs)));
 }
 #endif /* USE_COLOR */
 
 #ifdef USE_MOUSE
 struct mousedata {
-  MEVENT *mevent;
+    MEVENT *mevent;
 };
 
 static void
 no_mevent(void)
 {
-  rb_raise(rb_eRuntimeError, "no such mouse event");
+    rb_raise(rb_eRuntimeError, "no such mouse event");
 }
 
 #define GetMOUSE(obj, data) do {\
@@ -634,49 +793,53 @@ no_mevent(void)
 static void
 curses_mousedata_free(struct mousedata *mdata)
 {
-  if (mdata->mevent)
-    free(mdata->mevent);
+    if (mdata->mevent)
+	xfree(mdata->mevent);
 }
 
 static VALUE
 curses_getmouse(VALUE obj)
 {
-  struct mousedata *mdata;
-  VALUE val;
+    struct mousedata *mdata;
+    VALUE val;
 
-  val = Data_Make_Struct(cMouseEvent,struct mousedata,
-			 0,curses_mousedata_free,mdata);
-  mdata->mevent = (MEVENT*)xmalloc(sizeof(MEVENT));
-  return (getmouse(mdata->mevent) == OK) ? val : Qnil;
+    curses_stdscr();
+    val = Data_Make_Struct(cMouseEvent,struct mousedata,
+			   0,curses_mousedata_free,mdata);
+    mdata->mevent = (MEVENT*)xmalloc(sizeof(MEVENT));
+    return (getmouse(mdata->mevent) == OK) ? val : Qnil;
 }
 
 static VALUE
 curses_ungetmouse(VALUE obj, VALUE mevent)
 {
-  struct mousedata *mdata;
+    struct mousedata *mdata;
 
-  GetMOUSE(mevent,mdata);
-  return (ungetmouse(mdata->mevent) == OK) ? Qtrue : Qfalse;
+    curses_stdscr();
+    GetMOUSE(mevent,mdata);
+    return (ungetmouse(mdata->mevent) == OK) ? Qtrue : Qfalse;
 }
 
 static VALUE
 curses_mouseinterval(VALUE obj, VALUE interval)
 {
-  return mouseinterval(NUM2INT(interval)) ? Qtrue : Qfalse;
+    curses_stdscr();
+    return mouseinterval(NUM2INT(interval)) ? Qtrue : Qfalse;
 }
 
 static VALUE
 curses_mousemask(VALUE obj, VALUE mask)
 {
-  return INT2NUM(mousemask(NUM2UINT(mask),NULL));
+    curses_stdscr();
+    return INT2NUM(mousemask(NUM2UINT(mask),NULL));
 }
 
 #define DEFINE_MOUSE_GET_MEMBER(func_name,mem) \
 static VALUE func_name (VALUE mouse) \
 { \
-  struct mousedata *mdata; \
-  GetMOUSE(mouse, mdata); \
-  return (UINT2NUM(mdata->mevent -> mem)); \
+    struct mousedata *mdata; \
+    GetMOUSE(mouse, mdata); \
+    return (UINT2NUM(mdata->mevent -> mem)); \
 }
 
 DEFINE_MOUSE_GET_MEMBER(curs_mouse_id, id)
@@ -687,36 +850,39 @@ DEFINE_MOUSE_GET_MEMBER(curs_mouse_bstate, bstate)
 #undef define_curs_mouse_member
 #endif /* USE_MOUSE */
 
+#ifdef HAVE_TIMEOUT
 static VALUE
 curses_timeout(VALUE obj, VALUE delay)
 {
-#ifdef HAVE_TIMEOUT
-  timeout(NUM2INT(delay));
-  return Qnil;
-#else
-    rb_notimplement();
-#endif
+    curses_stdscr();
+    timeout(NUM2INT(delay));
+    return Qnil;
 }
+#else
+#define curses_timeout rb_f_notimplement
+#endif
 
+#ifdef HAVE_DEF_PROG_MODE
 static VALUE
 curses_def_prog_mode(VALUE obj)
 {
-#ifdef HAVE_DEF_PROG_MODE
-  return def_prog_mode() == OK ? Qtrue : Qfalse;
-#else
-    rb_notimplement();
-#endif
+    curses_stdscr();
+    return def_prog_mode() == OK ? Qtrue : Qfalse;
 }
+#else
+#define curses_def_prog_mode rb_f_notimplement
+#endif
 
+#ifdef HAVE_RESET_PROG_MODE
 static VALUE
 curses_reset_prog_mode(VALUE obj)
 {
-#ifdef HAVE_RESET_PROG_MODE
-  return reset_prog_mode() == OK ? Qtrue : Qfalse;
-#else
-    rb_notimplement();
-#endif
+    curses_stdscr();
+    return reset_prog_mode() == OK ? Qtrue : Qfalse;
 }
+#else
+#define curses_reset_prog_mode rb_f_notimplement
+#endif
 
 /*-------------------------- class Window --------------------------*/
 
@@ -772,7 +938,7 @@ static VALUE
 window_close(VALUE obj)
 {
     struct windata *winp;
-    
+
     GetWINDOW(obj, winp);
     delwin(winp->window);
     winp->window = 0;
@@ -785,10 +951,10 @@ static VALUE
 window_clear(VALUE obj)
 {
     struct windata *winp;
-    
+
     GetWINDOW(obj, winp);
     wclear(winp->window);
-    
+
     return Qnil;
 }
 
@@ -797,10 +963,10 @@ static VALUE
 window_clrtoeol(VALUE obj)
 {
     struct windata *winp;
-    
+
     GetWINDOW(obj, winp);
     wclrtoeol(winp->window);
-    
+
     return Qnil;
 }
 
@@ -809,10 +975,10 @@ static VALUE
 window_refresh(VALUE obj)
 {
     struct windata *winp;
-    
+
     GetWINDOW(obj, winp);
     wrefresh(winp->window);
-    
+
     return Qnil;
 }
 
@@ -837,7 +1003,7 @@ static VALUE
 window_move(VALUE obj, VALUE y, VALUE x)
 {
     struct windata *winp;
-    
+
     GetWINDOW(obj, winp);
     mvwin(winp->window, NUM2INT(y), NUM2INT(x));
 
@@ -849,7 +1015,7 @@ static VALUE
 window_setpos(VALUE obj, VALUE y, VALUE x)
 {
     struct windata *winp;
-    
+
     GetWINDOW(obj, winp);
     wmove(winp->window, NUM2INT(y), NUM2INT(x));
     return Qnil;
@@ -955,7 +1121,7 @@ window_begx(VALUE obj)
 static VALUE
 window_box(int argc, VALUE *argv, VALUE self)
 {
-    struct windata *winp; 
+    struct windata *winp;
     VALUE vert, hor, corn;
 
     rb_scan_args(argc, argv, "21", &vert, &hor, &corn);
@@ -964,24 +1130,24 @@ window_box(int argc, VALUE *argv, VALUE self)
     box(winp->window, NUM2CH(vert), NUM2CH(hor));
 
     if (!NIL_P(corn)) {
-      int cur_x, cur_y, x, y;
-      chtype c;
+	int cur_x, cur_y, x, y;
+	chtype c;
 
-      c = NUM2CH(corn);
-      getyx(winp->window, cur_y, cur_x);
-      x = NUM2INT(window_maxx(self)) - 1;
-      y = NUM2INT(window_maxy(self)) - 1;
-      wmove(winp->window, 0, 0);
-      waddch(winp->window, c);
-      wmove(winp->window, y, 0);
-      waddch(winp->window, c);
-      wmove(winp->window, y, x);
-      waddch(winp->window, c);
-      wmove(winp->window, 0, x);
-      waddch(winp->window, c);
-      wmove(winp->window, cur_y, cur_x);
+	c = NUM2CH(corn);
+	getyx(winp->window, cur_y, cur_x);
+	x = NUM2INT(window_maxx(self)) - 1;
+	y = NUM2INT(window_maxy(self)) - 1;
+	wmove(winp->window, 0, 0);
+	waddch(winp->window, c);
+	wmove(winp->window, y, 0);
+	waddch(winp->window, c);
+	wmove(winp->window, y, x);
+	waddch(winp->window, c);
+	wmove(winp->window, 0, x);
+	waddch(winp->window, c);
+	wmove(winp->window, cur_y, cur_x);
     }
-    
+
     return Qnil;
 }
 
@@ -990,7 +1156,7 @@ static VALUE
 window_standout(VALUE obj)
 {
     struct windata *winp;
-    
+
     GetWINDOW(obj, winp);
     wstandout(winp->window);
     return Qnil;
@@ -1001,7 +1167,7 @@ static VALUE
 window_standend(VALUE obj)
 {
     struct windata *winp;
-    
+
     GetWINDOW(obj, winp);
     wstandend(winp->window);
     return Qnil;
@@ -1012,7 +1178,7 @@ static VALUE
 window_inch(VALUE obj)
 {
     struct windata *winp;
-    
+
     GetWINDOW(obj, winp);
     return CH2FIX(winch(winp->window));
 }
@@ -1022,10 +1188,10 @@ static VALUE
 window_addch(VALUE obj, VALUE ch)
 {
     struct windata *winp;
-    
+
     GetWINDOW(obj, winp);
     waddch(winp->window, NUM2CH(ch));
-    
+
     return Qnil;
 }
 
@@ -1034,10 +1200,10 @@ static VALUE
 window_insch(VALUE obj, VALUE ch)
 {
     struct windata *winp;
-    
+
     GetWINDOW(obj, winp);
     winsch(winp->window, NUM2CH(ch));
-    
+
     return Qnil;
 }
 
@@ -1048,8 +1214,10 @@ window_addstr(VALUE obj, VALUE str)
     if (!NIL_P(str)) {
 	struct windata *winp;
 
+	StringValue(str);
+	str = rb_str_export_locale(str);
 	GetWINDOW(obj, winp);
-	waddstr(winp->window, STR2CSTR(str));
+	waddstr(winp->window, StringValueCStr(str));
     }
     return Qnil;
 }
@@ -1062,15 +1230,55 @@ window_addstr2(VALUE obj, VALUE str)
     return obj;
 }
 
+struct wgetch_arg {
+    WINDOW *win;
+    int c;
+};
+
+static VALUE
+wgetch_func(void *_arg)
+{
+    struct wgetch_arg *arg = (struct wgetch_arg *)_arg;
+    arg->c = wgetch(arg->win);
+    return Qnil;
+}
+
 /* def getch */
 static VALUE
 window_getch(VALUE obj)
 {
     struct windata *winp;
-    
-    rb_read_check(stdin);
+    struct wgetch_arg arg;
+    int c;
+
     GetWINDOW(obj, winp);
-    return UINT2NUM(wgetch(winp->window));
+    arg.win = winp->window;
+    rb_thread_blocking_region(wgetch_func, (void *)&arg, RUBY_UBF_IO, 0);
+    c = arg.c;
+    if (c == EOF) return Qnil;
+    if (rb_isprint(c)) {
+	char ch = (char)c;
+
+	return rb_locale_str_new(&ch, 1);
+    }
+    return UINT2NUM(c);
+}
+
+struct wgetstr_arg {
+    WINDOW *win;
+    char rtn[GETSTR_BUF_SIZE];
+};
+
+static VALUE
+wgetstr_func(void *_arg)
+{
+    struct wgetstr_arg *arg = (struct wgetstr_arg *)_arg;
+#if defined(HAVE_WGETNSTR)
+    wgetnstr(arg->win, arg->rtn, GETSTR_BUF_SIZE-1);
+#else
+    wgetstr(arg->win, arg->rtn);
+#endif
+    return Qnil;
 }
 
 /* def getstr */
@@ -1078,16 +1286,12 @@ static VALUE
 window_getstr(VALUE obj)
 {
     struct windata *winp;
-    char rtn[1024]; /* This should be big enough.. I hope */
-    
+    struct wgetstr_arg arg;
+
     GetWINDOW(obj, winp);
-    rb_read_check(stdin);
-#if defined(HAVE_WGETNSTR)
-    wgetnstr(winp->window, rtn, 1023);
-#else
-    wgetstr(winp->window, rtn);
-#endif
-    return rb_tainted_str_new2(rtn);
+    arg.win = winp->window;
+    rb_thread_blocking_region(wgetstr_func, (void *)&arg, RUBY_UBF_IO, 0);
+    return rb_locale_str_new_cstr(arg.rtn);
 }
 
 /* def delch */
@@ -1095,7 +1299,7 @@ static VALUE
 window_delch(VALUE obj)
 {
     struct windata *winp;
-    
+
     GetWINDOW(obj, winp);
     wdelch(winp->window);
     return Qnil;
@@ -1107,7 +1311,7 @@ window_deleteln(VALUE obj)
 {
 #if defined(HAVE_WDELETELN) || defined(wdeleteln)
     struct windata *winp;
-    
+
     GetWINDOW(obj, winp);
     wdeleteln(winp->window);
 #endif
@@ -1120,7 +1324,7 @@ window_insertln(VALUE obj)
 {
 #if defined(HAVE_WINSERTLN) || defined(winsertln)
     struct windata *winp;
-    
+
     GetWINDOW(obj, winp);
     winsertln(winp->window);
 #endif
@@ -1130,73 +1334,73 @@ window_insertln(VALUE obj)
 static VALUE
 window_scrollok(VALUE obj, VALUE bf)
 {
-  struct windata *winp;
+    struct windata *winp;
 
-  GetWINDOW(obj, winp);
-  scrollok(winp->window, RTEST(bf) ? TRUE : FALSE);
-  return Qnil;
+    GetWINDOW(obj, winp);
+    scrollok(winp->window, RTEST(bf) ? TRUE : FALSE);
+    return Qnil;
 }
 
 static VALUE
 window_idlok(VALUE obj, VALUE bf)
 {
-  struct windata *winp;
+    struct windata *winp;
 
-  GetWINDOW(obj, winp);
-  idlok(winp->window, RTEST(bf) ? TRUE : FALSE);
-  return Qnil;
+    GetWINDOW(obj, winp);
+    idlok(winp->window, RTEST(bf) ? TRUE : FALSE);
+    return Qnil;
 }
 
 static VALUE
 window_setscrreg(VALUE obj, VALUE top, VALUE bottom)
 {
 #ifdef HAVE_WSETSCRREG
-  struct windata *winp;
-  int res;
+    struct windata *winp;
+    int res;
 
-  GetWINDOW(obj, winp);
-  res = wsetscrreg(winp->window, NUM2INT(top), NUM2INT(bottom));
-  /* may have to raise exception on ERR */
-  return (res == OK) ? Qtrue : Qfalse;
+    GetWINDOW(obj, winp);
+    res = wsetscrreg(winp->window, NUM2INT(top), NUM2INT(bottom));
+    /* may have to raise exception on ERR */
+    return (res == OK) ? Qtrue : Qfalse;
 #else
-  return Qfalse;
+    return Qfalse;
 #endif
 }
 
 #if defined(USE_COLOR) && defined(HAVE_WCOLOR_SET)
 static VALUE
-window_color_set(VALUE obj, VALUE col) 
+window_color_set(VALUE obj, VALUE col)
 {
-  struct windata *winp;
-  int res;
+    struct windata *winp;
+    int res;
 
-  GetWINDOW(obj, winp);
-  res = wcolor_set(winp->window, NUM2INT(col), NULL);
-  return (res == OK) ? Qtrue : Qfalse;
+    GetWINDOW(obj, winp);
+    res = wcolor_set(winp->window, NUM2INT(col), NULL);
+    return (res == OK) ? Qtrue : Qfalse;
 }
 #endif /* defined(USE_COLOR) && defined(HAVE_WCOLOR_SET) */
 
 static VALUE
 window_scroll(VALUE obj)
 {
-  struct windata *winp;
+    struct windata *winp;
 
-  GetWINDOW(obj, winp);
-  /* may have to raise exception on ERR */
-  return (scroll(winp->window) == OK) ? Qtrue : Qfalse;
+    GetWINDOW(obj, winp);
+    /* may have to raise exception on ERR */
+    return (scroll(winp->window) == OK) ? Qtrue : Qfalse;
 }
 
 static VALUE
 window_scrl(VALUE obj, VALUE n)
 {
 #ifdef HAVE_WSCRL
-  struct windata *winp;
+    struct windata *winp;
 
-  GetWINDOW(obj, winp);
-  /* may have to raise exception on ERR */
-  return (wscrl(winp->window,NUM2INT(n)) == OK) ? Qtrue : Qfalse;
+    GetWINDOW(obj, winp);
+    /* may have to raise exception on ERR */
+    return (wscrl(winp->window,NUM2INT(n)) == OK) ? Qtrue : Qfalse;
 #else
-  return Qfalse;
+    return Qfalse;
 #endif
 }
 
@@ -1204,12 +1408,12 @@ static VALUE
 window_attroff(VALUE obj, VALUE attrs)
 {
 #ifdef HAVE_WATTROFF
-  struct windata *winp;
+    struct windata *winp;
 
-  GetWINDOW(obj,winp);
-  return INT2FIX(wattroff(winp->window,NUM2INT(attrs)));
+    GetWINDOW(obj,winp);
+    return INT2FIX(wattroff(winp->window,NUM2INT(attrs)));
 #else
-  return Qtrue;
+    return Qtrue;
 #endif
 }
 
@@ -1217,21 +1421,21 @@ static VALUE
 window_attron(VALUE obj, VALUE attrs)
 {
 #ifdef HAVE_WATTRON
-  struct windata *winp;
-  VALUE val;
+    struct windata *winp;
+    VALUE val;
 
-  GetWINDOW(obj,winp);
-  val = INT2FIX(wattron(winp->window,NUM2INT(attrs)));
-  if( rb_block_given_p() ){
-    rb_yield(val);
-    wattroff(winp->window,NUM2INT(attrs));
-    return val;
-  }
-  else{
-    return val;
-  }
+    GetWINDOW(obj,winp);
+    val = INT2FIX(wattron(winp->window,NUM2INT(attrs)));
+    if (rb_block_given_p()) {
+	rb_yield(val);
+	wattroff(winp->window,NUM2INT(attrs));
+	return val;
+    }
+    else{
+	return val;
+    }
 #else
-  return Qtrue;
+    return Qtrue;
 #endif
 }
 
@@ -1239,12 +1443,12 @@ static VALUE
 window_attrset(VALUE obj, VALUE attrs)
 {
 #ifdef HAVE_WATTRSET
-  struct windata *winp;
+    struct windata *winp;
 
-  GetWINDOW(obj,winp);
-  return INT2FIX(wattrset(winp->window,NUM2INT(attrs)));
+    GetWINDOW(obj,winp);
+    return INT2FIX(wattrset(winp->window,NUM2INT(attrs)));
 #else
-  return Qtrue;
+    return Qtrue;
 #endif
 }
 
@@ -1252,24 +1456,24 @@ static VALUE
 window_bkgdset(VALUE obj, VALUE ch)
 {
 #ifdef HAVE_WBKGDSET
-  struct windata *winp;
+    struct windata *winp;
 
-  GetWINDOW(obj,winp);
-  wbkgdset(winp->window, NUM2CH(ch));
+    GetWINDOW(obj,winp);
+    wbkgdset(winp->window, NUM2CH(ch));
 #endif
-  return Qnil;
+    return Qnil;
 }
 
 static VALUE
 window_bkgd(VALUE obj, VALUE ch)
 {
 #ifdef HAVE_WBKGD
-  struct windata *winp;
+    struct windata *winp;
 
-  GetWINDOW(obj,winp);
-  return (wbkgd(winp->window, NUM2CH(ch)) == OK) ? Qtrue : Qfalse;
+    GetWINDOW(obj,winp);
+    return (wbkgd(winp->window, NUM2CH(ch)) == OK) ? Qtrue : Qfalse;
 #else
-  return Qfalse;
+    return Qfalse;
 #endif
 }
 
@@ -1277,13 +1481,13 @@ static VALUE
 window_getbkgd(VALUE obj)
 {
 #ifdef HAVE_WGETBKGD
-  chtype c;
-  struct windata *winp;
+    chtype c;
+    struct windata *winp;
 
-  GetWINDOW(obj,winp);
-  return (c = getbkgd(winp->window) != ERR) ? CH2FIX(c) : Qnil;
+    GetWINDOW(obj,winp);
+    return (c = getbkgd(winp->window) != ERR) ? CH2FIX(c) : Qnil;
 #else
-  return Qnil;
+    return Qnil;
 #endif
 }
 
@@ -1291,69 +1495,69 @@ static VALUE
 window_resize(VALUE obj, VALUE lin, VALUE col)
 {
 #if defined(HAVE_WRESIZE)
-  struct windata *winp;
+    struct windata *winp;
 
-  GetWINDOW(obj,winp);
-  return wresize(winp->window, NUM2INT(lin), NUM2INT(col)) == OK ? Qtrue : Qfalse;
+    GetWINDOW(obj,winp);
+    return wresize(winp->window, NUM2INT(lin), NUM2INT(col)) == OK ? Qtrue : Qfalse;
 #else
-  return Qnil;
+    return Qnil;
 #endif
 }
 
 
+#ifdef HAVE_KEYPAD
 static VALUE
 window_keypad(VALUE obj, VALUE val)
 {
-#ifdef HAVE_KEYPAD
-  struct windata *winp;
+    struct windata *winp;
 
-  GetWINDOW(obj,winp);
-  /* keypad() of NetBSD's libcurses returns no value */
+    GetWINDOW(obj,winp);
+    /* keypad() of NetBSD's libcurses returns no value */
 #if defined(__NetBSD__) && !defined(NCURSES_VERSION)
-  keypad(winp->window,(RTEST(val) ? TRUE : FALSE));
-  return Qnil;
+    keypad(winp->window,(RTEST(val) ? TRUE : FALSE));
+    return Qnil;
 #else
-  /* may have to raise exception on ERR */
-  return (keypad(winp->window,RTEST(val) ? TRUE : FALSE)) == OK ?
-    Qtrue : Qfalse;
+    /* may have to raise exception on ERR */
+    return (keypad(winp->window,RTEST(val) ? TRUE : FALSE)) == OK ?
+	Qtrue : Qfalse;
 #endif
-#else
-    rb_notimplement();
-#endif /* HAVE_KEYPAD */
 }
+#else
+#define window_keypad rb_f_notimplement
+#endif
 
+#ifdef HAVE_NODELAY
 static VALUE
 window_nodelay(VALUE obj, VALUE val)
 {
-#ifdef HAVE_NODELAY
-  struct windata *winp;
-  GetWINDOW(obj,winp);
+    struct windata *winp;
+    GetWINDOW(obj,winp);
 
-  /* nodelay() of NetBSD's libcurses returns no value */
+    /* nodelay() of NetBSD's libcurses returns no value */
 #if defined(__NetBSD__) && !defined(NCURSES_VERSION)
-  nodelay(winp->window, RTEST(val) ? TRUE : FALSE);
-  return Qnil;
+    nodelay(winp->window, RTEST(val) ? TRUE : FALSE);
+    return Qnil;
 #else
-  return nodelay(winp->window,RTEST(val) ? TRUE : FALSE) == OK ? Qtrue : Qfalse;
-#endif
-#else
-    rb_notimplement();
+    return nodelay(winp->window,RTEST(val) ? TRUE : FALSE) == OK ? Qtrue : Qfalse;
 #endif
 }
+#else
+#define window_nodelay rb_f_notimplement
+#endif
 
+#ifdef HAVE_WTIMEOUT
 static VALUE
 window_timeout(VALUE obj, VALUE delay)
 {
-#ifdef HAVE_WTIMEOUT
-  struct windata *winp;
-  GetWINDOW(obj,winp);
+    struct windata *winp;
+    GetWINDOW(obj,winp);
 
-  wtimeout(winp->window,NUM2INT(delay));
-  return Qnil;
-#else
-    rb_notimplement();
-#endif
+    wtimeout(winp->window,NUM2INT(delay));
+    return Qnil;
 }
+#else
+#define window_timeout rb_f_notimplement
+#endif
 
 /*------------------------- Initialization -------------------------*/
 void
@@ -1374,6 +1578,12 @@ Init_curses(void)
     rb_define_method(cMouseEvent, "bstate", curs_mouse_bstate, 0);
 #endif /* USE_MOUSE */
 
+    rb_define_module_function(mCurses, "ESCDELAY=", curses_escdelay_set, 1);
+    rb_define_module_function(mCurses, "ESCDELAY", curses_escdelay_get, 0);
+    rb_define_module_function(mCurses, "TABSIZE", curses_tabsize_get, 0);
+    rb_define_module_function(mCurses, "TABSIZE=", curses_tabsize_set, 1);
+
+    rb_define_module_function(mCurses, "use_default_colors", curses_use_default_colors, 0);
     rb_define_module_function(mCurses, "init_screen", curses_init_screen, 0);
     rb_define_module_function(mCurses, "close_screen", curses_close_screen, 0);
     rb_define_module_function(mCurses, "closed?", curses_closed, 0);
@@ -1388,8 +1598,8 @@ Init_curses(void)
     rb_define_module_function(mCurses, "noraw", curses_noraw, 0);
     rb_define_module_function(mCurses, "cbreak", curses_cbreak, 0);
     rb_define_module_function(mCurses, "nocbreak", curses_nocbreak, 0);
-    rb_define_alias(mCurses, "crmode", "cbreak");
-    rb_define_alias(mCurses, "nocrmode", "nocbreak");
+    rb_define_module_function(mCurses, "crmode", curses_nocbreak, 0);
+    rb_define_module_function(mCurses, "nocrmode", curses_nocbreak, 0);
     rb_define_module_function(mCurses, "nl", curses_nl, 0);
     rb_define_module_function(mCurses, "nonl", curses_nonl, 0);
     rb_define_module_function(mCurses, "beep", curses_beep, 0);
@@ -1427,7 +1637,9 @@ Init_curses(void)
     rb_define_module_function(mCurses, "has_colors?", curses_has_colors, 0);
     rb_define_module_function(mCurses, "can_change_color?",
 			      curses_can_change_color, 0);
+    rb_define_module_function(mCurses, "colors", curses_colors, 0);
     rb_define_module_function(mCurses, "color_content", curses_color_content, 1);
+    rb_define_module_function(mCurses, "color_pairs", curses_color_pairs, 0);
     rb_define_module_function(mCurses, "pair_content", curses_pair_content, 1);
     rb_define_module_function(mCurses, "color_pair", curses_color_pair, 1);
     rb_define_module_function(mCurses, "pair_number", curses_pair_number, 1);
@@ -1663,14 +1875,14 @@ Init_curses(void)
 #ifdef KEY_F
     /* KEY_F(n) : 0 <= n <= 63 */
     {
-      int i;
-      char c[8];
-      for( i=0; i<64; i++ ){
-	sprintf(c, "KEY_F%d", i);
-	rb_define_const(mCurses, c, INT2NUM(KEY_F(i)));
-	sprintf(c, "F%d", i);
-	rb_define_const(mKey, c, INT2NUM(KEY_F(i)));
-      }
+	int i;
+	char c[8];
+	for (i=0; i<64; i++) {
+	    sprintf(c, "KEY_F%d", i);
+	    rb_define_const(mCurses, c, INT2NUM(KEY_F(i)));
+	    sprintf(c, "F%d", i);
+	    rb_define_const(mKey, c, INT2NUM(KEY_F(i)));
+	}
     }
 #endif
 #ifdef KEY_DL
@@ -2006,12 +2218,12 @@ Init_curses(void)
     rb_define_const(mKey, "MAX", INT2NUM(KEY_MAX));
 #endif
     {
-      int c;
-      char name[] = "KEY_CTRL_x";
-      for( c = 'A'; c <= 'Z'; c++ ){
-	sprintf(name, "KEY_CTRL_%c", c);
-	rb_define_const(mCurses, name, INT2FIX(c - 'A' + 1));
-      }
+	int c;
+	char name[] = "KEY_CTRL_x";
+	for (c = 'A'; c <= 'Z'; c++) {
+	    name[sizeof(name) - 2] = c;
+	    rb_define_const(mCurses, name, INT2FIX(c - 'A' + 1));
+	}
     }
 #undef rb_curses_define_const
 
