@@ -20,11 +20,10 @@
 #define RTLD_NOW 0
 #endif
 #else
-# if defined(HAVE_WINDOWS_H)
+# if defined(_WIN32)
 #   include <windows.h>
-#   define dlclose(ptr) FreeLibrary((HINSTANCE)ptr)
 #   define dlopen(name,flag) ((void*)LoadLibrary(name))
-#   define dlerror()    "unknown error"
+#   define dlerror() strerror(rb_w32_map_errno(GetLastError()))
 #   define dlsym(handle,name) ((void*)GetProcAddress(handle,name))
 #   define RTLD_LAZY -1
 #   define RTLD_NOW  -1
@@ -33,7 +32,7 @@
 #endif
 
 #define MAX_CALLBACK 5
-#define DLSTACK_TYPE long
+#define DLSTACK_TYPE SIGNED_VALUE
 #define DLSTACK_SIZE (20)
 #define DLSTACK_PROTO \
     DLSTACK_TYPE,DLSTACK_TYPE,DLSTACK_TYPE,DLSTACK_TYPE,DLSTACK_TYPE,\
@@ -217,6 +216,7 @@ struct ptr_data {
 #define RPTR_DATA(obj) ((struct ptr_data *)(DATA_PTR(obj)))
 
 VALUE rb_dlcfunc_new(void (*func)(), int dltype, const char * name, ID calltype);
+int rb_dlcfunc_kind_p(VALUE func);
 VALUE rb_dlptr_new(void *ptr, long size, freefunc_t func);
 VALUE rb_dlptr_new2(VALUE klass, void *ptr, long size, freefunc_t func);
 VALUE rb_dlptr_malloc(long size, freefunc_t func);
