@@ -1,8 +1,9 @@
 begin
   require "openssl"
-  require File.join(File.dirname(__FILE__), "utils.rb")
 rescue LoadError
 end
+require "digest/md5"
+require "test/unit"
 
 if defined?(OpenSSL)
 
