@@ -19,94 +19,36 @@ class TestIO < Test::Unit::TestCase
   end
 
   def have_nonblock?
-    IO.instance_methods.index(:"nonblock=")
+    IO.method_defined?("nonblock=")
   end
 
-  def test_gets_rs
-    # default_rs
+  def pipe(wp, rp)
+    re, we = nil, nil
     r, w = IO.pipe
-    w.print "aaa\nbbb\n"
-    w.close
-    assert_equal "aaa\n", r.gets
-    assert_equal "bbb\n", r.gets
-    assert_nil r.gets
-    r.close
-
-    # nil
-    r, w = IO.pipe
-    w.print "a\n\nb\n\n"
-    w.close
-    assert_equal "a\n\nb\n\n", r.gets(nil)
-    assert_nil r.gets("")
-    r.close
-
-    # "\377"
-    r, w = IO.pipe('ascii-8bit')
-    w.print "\377xyz"
-    w.close
-    r.binmode
-    assert_equal("\377", r.gets("\377"), "[ruby-dev:24460]")
-    r.close
-
-    # ""
-    r, w = IO.pipe
-    w.print "a\n\nb\n\n"
-    w.close
-    assert_equal "a\n\n", r.gets(""), "[ruby-core:03771]"
-    assert_equal "b\n\n", r.gets("")
-    assert_nil r.gets("")
-    r.close
-  end
-
-  def test_gets_limit_extra_arg
-    with_pipe {|r, w|
-      r, w = IO.pipe
-      w << "0123456789\n0123456789"
-      w.close
-      assert_equal("0123456789\n0", r.gets(nil, 12))
-      assert_raise(TypeError) { r.gets(3,nil) }
-    }
-  end
-
-  # This test cause SEGV.
-  def test_ungetc
-    r, w = IO.pipe
-    w.close
-    assert_raise(IOError, "[ruby-dev:31650]") { 20000.times { r.ungetc "a" } }
-  ensure
-    r.close
-  end
-
-  def test_ungetbyte
-    t = make_tempfile
-    t.open
-    t.ungetbyte(0x41)
-    assert_equal(0x41, t.getbyte)
-    t.rewind
-    t.ungetbyte("qux")
-    assert_equal("quxfoo\n", t.gets)
-    t.set_encoding("utf-8")
-    t.ungetbyte(0x89)
-    t.ungetbyte(0x8e)
-    t.ungetbyte("\xe7")
-    t.ungetbyte("\xe7\xb4\x85")
-    assert_equal("\u7d05\u7389bar\n", t.gets)
-  end
-
-  def test_each_byte
-    r, w = IO.pipe
-    w << "abc def"
-    w.close
-    r.each_byte {|byte| break if byte == 32 }
-    assert_equal("def", r.read, "[ruby-dev:31659]")
-  ensure
-    r.close
-  end
-
-  def test_rubydev33072
-    assert_raise(Errno::ENOENT, "[ruby-dev:33072]") do
-      File.read("empty", nil, nil, {})
+    rt = Thread.new do
+      begin
+        rp.call(r)
+      rescue Exception
+        r.close
+        re = $!
+      end
     end
+    wt = Thread.new do
+      begin
+        wp.call(w)
+      rescue Exception
+        w.close
+        we = $!
+      end
+    end
+    flunk("timeout") unless wt.join(10) && rt.join(10)
+  ensure
+    w.close unless !w || w.closed?
+    r.close unless !r || r.closed?
+    (wt.kill; wt.join) if wt
+    (rt.kill; rt.join) if rt
+    raise we if we
+    raise re if re
   end
 
   def with_pipe
@@ -120,14 +62,12 @@ class TestIO < Test::Unit::TestCase
   end
 
   def with_read_pipe(content)
-    r, w = IO.pipe
-    w << content
-    w.close
-    begin
+    pipe(proc do |w|
+      w << content
+      w.close
+    end, proc do |r|
       yield r
-    ensure
-      r.close
-    end
+    end)
   end
 
   def mkcdtmpdir
@@ -138,6 +78,174 @@ class TestIO < Test::Unit::TestCase
     }
   end
 
+  def test_pipe
+    r, w = IO.pipe
+    assert_instance_of(IO, r)
+    assert_instance_of(IO, w)
+    [
+      Thread.start{
+        w.print "abc"
+        w.close
+      },
+      Thread.start{
+        assert_equal("abc", r.read)
+        r.close
+      }
+    ].each{|thr| thr.join}
+  end
+
+  def test_pipe_block
+    x = nil
+    ret = IO.pipe {|r, w|
+      x = [r,w]
+      assert_instance_of(IO, r)
+      assert_instance_of(IO, w)
+      [
+        Thread.start do
+          w.print "abc"
+          w.close
+        end,
+        Thread.start do
+          assert_equal("abc", r.read)
+        end
+      ].each{|thr| thr.join}
+      assert(!r.closed?)
+      assert(w.closed?)
+      :foooo
+    }
+    assert_equal(:foooo, ret)
+    assert(x[0].closed?)
+    assert(x[1].closed?)
+  end
+
+  def test_pipe_block_close
+    4.times {|i|
+      x = nil
+      IO.pipe {|r, w|
+        x = [r,w]
+        r.close if (i&1) == 0
+        w.close if (i&2) == 0
+      }
+      assert(x[0].closed?)
+      assert(x[1].closed?)
+    }
+  end
+
+  def test_gets_rs
+    # default_rs
+    pipe(proc do |w|
+      w.print "aaa\nbbb\n"
+      w.close
+    end, proc do |r|
+      assert_equal "aaa\n", r.gets
+      assert_equal "bbb\n", r.gets
+      assert_nil r.gets
+      r.close
+    end)
+
+    # nil
+    pipe(proc do |w|
+      w.print "a\n\nb\n\n"
+      w.close
+    end, proc do |r|
+      assert_equal "a\n\nb\n\n", r.gets(nil)
+      assert_nil r.gets("")
+      r.close
+    end)
+
+    # "\377"
+    pipe(proc do |w|
+      w.print "\377xyz"
+      w.close
+    end, proc do |r|
+      r.binmode
+      assert_equal("\377", r.gets("\377"), "[ruby-dev:24460]")
+      r.close
+    end)
+
+    # ""
+    pipe(proc do |w|
+      w.print "a\n\nb\n\n"
+      w.close
+    end, proc do |r|
+      assert_equal "a\n\n", r.gets(""), "[ruby-core:03771]"
+      assert_equal "b\n\n", r.gets("")
+      assert_nil r.gets("")
+      r.close
+    end)
+  end
+
+  def test_gets_limit_extra_arg
+    pipe(proc do |w|
+      w << "0123456789\n0123456789"
+      w.close
+    end, proc do |r|
+      assert_equal("0123456789\n0", r.gets(nil, 12))
+      assert_raise(TypeError) { r.gets(3,nil) }
+    end)
+  end
+
+  # This test cause SEGV.
+  def test_ungetc
+    pipe(proc do |w|
+      w.close
+    end, proc do |r|
+      s = "a" * 1000
+      assert_raise(IOError, "[ruby-dev:31650]") { 200.times { r.ungetc s } }
+    end)
+  end
+
+  def test_ungetbyte
+    t = make_tempfile
+    t.open
+    t.binmode
+    t.ungetbyte(0x41)
+    assert_equal(-1, t.pos)
+    assert_equal(0x41, t.getbyte)
+    t.rewind
+    assert_equal(0, t.pos)
+    t.ungetbyte("qux")
+    assert_equal(-3, t.pos)
+    assert_equal("quxfoo\n", t.gets)
+    assert_equal(4, t.pos)
+    t.set_encoding("utf-8")
+    t.ungetbyte(0x89)
+    t.ungetbyte(0x8e)
+    t.ungetbyte("\xe7")
+    t.ungetbyte("\xe7\xb4\x85")
+    assert_equal(-2, t.pos)
+    assert_equal("\u7d05\u7389bar\n", t.gets)
+  end
+
+  def test_each_byte
+    pipe(proc do |w|
+      w << "abc def"
+      w.close
+    end, proc do |r|
+      r.each_byte {|byte| break if byte == 32 }
+      assert_equal("def", r.read, "[ruby-dev:31659]")
+    end)
+  end
+
+  def test_each_codepoint
+    t = make_tempfile
+    bug2959 = '[ruby-core:28650]'
+    a = ""
+    File.open(t, 'rt') {|f|
+      f.each_codepoint {|c| a << c}
+    }
+    assert_equal("foo\nbar\nbaz\n", a, bug2959)
+  end
+
+  def test_rubydev33072
+    t = make_tempfile
+    path = t.path
+    t.close!
+    assert_raise(Errno::ENOENT, "[ruby-dev:33072]") do
+      File.read(path, nil, nil, {})
+    end
+  end
+
   def test_copy_stream
     mkcdtmpdir {
 
@@ -174,12 +282,13 @@ class TestIO < Test::Unit::TestCase
         IO.copy_stream("src", "nodir/bar")
       }
 
-      with_pipe {|r, w|
+      pipe(proc do |w|
         ret = IO.copy_stream("src", w)
         assert_equal(content.bytesize, ret)
         w.close
+      end, proc do |r|
         assert_equal(content, r.read)
-      }
+      end)
 
       with_pipe {|r, w|
         w.close
@@ -195,83 +304,96 @@ class TestIO < Test::Unit::TestCase
 
       with_read_pipe("abc") {|r1|
         assert_equal("a", r1.getc)
-        with_pipe {|r2, w2|
+        pipe(proc do |w2|
           w2.sync = false
           w2 << "def"
           ret = IO.copy_stream(r1, w2)
           assert_equal(2, ret)
           w2.close
+        end, proc do |r2|
           assert_equal("defbc", r2.read)
-        }
+        end)
       }
 
       with_read_pipe("abc") {|r1|
         assert_equal("a", r1.getc)
-        with_pipe {|r2, w2|
+        pipe(proc do |w2|
           w2.sync = false
           w2 << "def"
           ret = IO.copy_stream(r1, w2, 1)
           assert_equal(1, ret)
           w2.close
+        end, proc do |r2|
           assert_equal("defb", r2.read)
-        }
+        end)
       }
 
       with_read_pipe("abc") {|r1|
         assert_equal("a", r1.getc)
-        with_pipe {|r2, w2|
+        pipe(proc do |w2|
           ret = IO.copy_stream(r1, w2)
           assert_equal(2, ret)
           w2.close
+        end, proc do |r2|
           assert_equal("bc", r2.read)
-        }
+        end)
       }
 
       with_read_pipe("abc") {|r1|
         assert_equal("a", r1.getc)
-        with_pipe {|r2, w2|
+        pipe(proc do |w2|
           ret = IO.copy_stream(r1, w2, 1)
           assert_equal(1, ret)
           w2.close
+        end, proc do |r2|
           assert_equal("b", r2.read)
-        }
+        end)
       }
 
       with_read_pipe("abc") {|r1|
         assert_equal("a", r1.getc)
-        with_pipe {|r2, w2|
+        pipe(proc do |w2|
           ret = IO.copy_stream(r1, w2, 0)
           assert_equal(0, ret)
           w2.close
+        end, proc do |r2|
           assert_equal("", r2.read)
-        }
+        end)
       }
 
-      with_pipe {|r1, w1|
+      pipe(proc do |w1|
         w1 << "abc"
+        w1 << "def"
+        w1.close
+      end, proc do |r1|
         assert_equal("a", r1.getc)
-        with_pipe {|r2, w2|
-          w1 << "def"
-          w1.close
+        pipe(proc do |w2|
           ret = IO.copy_stream(r1, w2)
           assert_equal(5, ret)
           w2.close
+        end, proc do |r2|
           assert_equal("bcdef", r2.read)
-        }
-      }
+        end)
+      end)
 
-      with_pipe {|r, w|
+      pipe(proc do |w|
         ret = IO.copy_stream("src", w, 1, 1)
         assert_equal(1, ret)
         w.close
+      end, proc do |r|
         assert_equal(content[1,1], r.read)
-      }
+      end)
 
       if have_nonblock?
         with_read_pipe("abc") {|r1|
           assert_equal("a", r1.getc)
           with_pipe {|r2, w2|
-            w2.nonblock = true
+            begin
+              w2.nonblock = true
+            rescue Errno::EBADF
+              skip "nonblocking IO for pipe is not implemented"
+              break
+            end
             s = w2.syswrite("a" * 100000)
             t = Thread.new { sleep 0.1; r2.read }
             ret = IO.copy_stream(r1, w2)
@@ -325,10 +447,14 @@ class TestIO < Test::Unit::TestCase
       if have_nonblock?
         with_pipe {|r1, w1|
           with_pipe {|r2, w2|
+            begin
+              r1.nonblock = true
+              w2.nonblock = true
+            rescue Errno::EBADF
+              skip "nonblocking IO for pipe is not implemented"
+            end
             t1 = Thread.new { w1 << megacontent; w1.close }
             t2 = Thread.new { r2.read }
-            r1.nonblock = true
-            w2.nonblock = true
             ret = IO.copy_stream(r1, w2)
             assert_equal(megacontent.bytesize, ret)
             w2.close
@@ -363,15 +489,16 @@ class TestIO < Test::Unit::TestCase
   def test_copy_stream_rbuf
     mkcdtmpdir {
       begin
-        with_pipe {|r, w|
+        pipe(proc do |w|
           File.open("foo", "w") {|f| f << "abcd" }
           File.open("foo") {|f|
             f.read(1)
             assert_equal(3, IO.copy_stream(f, w, 10, 1))
           }
           w.close
+        end, proc do |r|
           assert_equal("bcd", r.read)
-        }
+        end)
       rescue NotImplementedError
         skip "pread(2) is not implemtented."
       end
@@ -455,8 +582,12 @@ class TestIO < Test::Unit::TestCase
 
       if have_nonblock?
         with_socketpair {|s1, s2|
+          begin
+            s1.nonblock = true
+          rescue Errno::EBADF
+            skip "nonblocking IO for pipe is not implemented"
+          end
           t = Thread.new { s2.read }
-          s1.nonblock = true
           ret = IO.copy_stream("megasrc", s1)
           assert_equal(megacontent.bytesize, ret)
           s1.close
@@ -621,19 +752,20 @@ class TestIO < Test::Unit::TestCase
   end
 
   def test_copy_stream_strio_rbuf
-    with_pipe {|r, w|
+    pipe(proc do |w|
       w << "abcd"
       w.close
+    end, proc do |r|
       assert_equal("a", r.read(1))
       sio = StringIO.new
       IO.copy_stream(r, sio)
       assert_equal("bcd", sio.string)
-    }
+    end)
   end
 
   def test_copy_stream_src_wbuf
     mkcdtmpdir {
-      with_pipe {|r, w|
+      pipe(proc do |w|
         File.open("foe", "w+") {|f|
           f.write "abcd\n"
           f.rewind
@@ -642,17 +774,19 @@ class TestIO < Test::Unit::TestCase
         }
         assert_equal("xycd\n", File.read("foe"))
         w.close
+      end, proc do |r|
         assert_equal("cd\n", r.read)
         r.close
-      }
+      end)
     }
   end
 
   def test_copy_stream_dst_rbuf
     mkcdtmpdir {
-      with_pipe {|r, w|
+      pipe(proc do |w|
         w << "xyz"
         w.close
+      end, proc do |r|
         File.open("fom", "w+b") {|f|
           f.write "abcd\n"
           f.rewind
@@ -661,29 +795,19 @@ class TestIO < Test::Unit::TestCase
           IO.copy_stream(r, f)
         }
         assert_equal("abxyz", File.read("fom"))
-      }
+      end)
     }
   end
 
   def safe_4
-    Thread.new do
-      Timeout.timeout(10) do
-        $SAFE = 4
-        yield
-      end
-    end.join
-  end
-
-  def pipe(wp, rp)
-    r, w = IO.pipe
-    rt = Thread.new { rp.call(r) }
-    wt = Thread.new { wp.call(w) }
-    flunk("timeout") unless rt.join(10) && wt.join(10)
-  ensure
-    r.close unless !r || r.closed?
-    w.close unless !w || w.closed?
-    (rt.kill; rt.join) if rt
-    (wt.kill; wt.join) if wt
+    t = Thread.new do
+      $SAFE = 4
+      yield
+    end
+    unless t.join(10)
+      t.kill
+      flunk("timeout in safe_4")
+    end
   end
 
   def ruby(*args)
@@ -757,7 +881,7 @@ class TestIO < Test::Unit::TestCase
 
   def test_inspect
     with_pipe do |r, w|
-      assert(r.inspect =~ /^#<IO:0x[0-9a-f]+>$/)
+      assert_match(/^#<IO:fd \d+>$/, r.inspect)
       assert_raise(SecurityError) do
         safe_4 { r.inspect }
       end
@@ -776,18 +900,29 @@ class TestIO < Test::Unit::TestCase
     end)
   end
 
-  def test_readpartial_error
+  def test_readpartial_lock
     with_pipe do |r, w|
       s = ""
       t = Thread.new { r.readpartial(5, s) }
       0 until s.size == 5
-      s.clear
+      assert_raise(RuntimeError) { s.clear }
       w.write "foobarbaz"
       w.close
-      assert_raise(RuntimeError) { t.join }
+      assert_equal("fooba", t.value)
     end
   end
 
+  def test_readpartial_pos
+    mkcdtmpdir {
+      open("foo", "w") {|f| f << "abc" }
+      open("foo") {|f|
+        f.seek(0)
+        assert_equal("ab", f.readpartial(2))
+        assert_equal(2, f.pos)
+      }
+    }
+  end
+
   def test_read
     pipe(proc do |w|
       w.write "foobarbaz"
@@ -800,15 +935,15 @@ class TestIO < Test::Unit::TestCase
     end)
   end
 
-  def test_read_error
+  def test_read_lock
     with_pipe do |r, w|
       s = ""
       t = Thread.new { r.read(5, s) }
       0 until s.size == 5
-      s.clear
+      assert_raise(RuntimeError) { s.clear }
       w.write "foobarbaz"
       w.close
-      assert_raise(RuntimeError) { t.join }
+      assert_equal("fooba", t.value)
     end
   end
 
@@ -822,6 +957,32 @@ class TestIO < Test::Unit::TestCase
     end)
   end
 
+  def test_read_nonblock_error
+    return if !have_nonblock?
+    skip "IO#read_nonblock is not supported on file/pipe." if /mswin|bccwin|mingw/ =~ RUBY_PLATFORM
+    with_pipe {|r, w|
+      begin
+        r.read_nonblock 4096
+      rescue Errno::EWOULDBLOCK
+        assert_kind_of(IO::WaitReadable, $!)
+      end
+    }
+  end
+
+  def test_write_nonblock_error
+    return if !have_nonblock?
+    skip "IO#write_nonblock is not supported on file/pipe." if /mswin|bccwin|mingw/ =~ RUBY_PLATFORM
+    with_pipe {|r, w|
+      begin
+        loop {
+          w.write_nonblock "a"*100000
+        }
+      rescue Errno::EWOULDBLOCK
+        assert_kind_of(IO::WaitWritable, $!)
+      end
+    }
+  end
+
   def test_gets
     pipe(proc do |w|
       w.write "foobarbaz"
@@ -1085,6 +1246,21 @@ class TestIO < Test::Unit::TestCase
     end
   end
 
+  def test_pos
+    t = make_tempfile
+
+    open(t.path, IO::RDWR|IO::CREAT|IO::TRUNC, 0600) do |f|
+      f.write "Hello"
+      assert_equal(5, f.pos)
+    end
+    open(t.path, IO::RDWR|IO::CREAT|IO::TRUNC, 0600) do |f|
+      f.sync = true
+      f.read
+      f.write "Hello"
+      assert_equal(5, f.pos)
+    end
+  end
+
   def test_sysseek
     t = make_tempfile
 
@@ -1135,13 +1311,13 @@ class TestIO < Test::Unit::TestCase
 
   def test_sysopen
     t = make_tempfile
-    
+
     fd = IO.sysopen(t.path)
     assert_kind_of(Integer, fd)
     f = IO.for_fd(fd)
     assert_equal("foo\nbar\nbaz\n", f.read)
     f.close
-    
+
     fd = IO.sysopen(t.path, "w", 0666)
     assert_kind_of(Integer, fd)
     if defined?(Fcntl::F_GETFL)
@@ -1151,7 +1327,7 @@ class TestIO < Test::Unit::TestCase
     end
     f.write("FOO\n")
     f.close
-    
+
     fd = IO.sysopen(t.path, "r")
     assert_kind_of(Integer, fd)
     f = IO.for_fd(fd)
@@ -1159,6 +1335,49 @@ class TestIO < Test::Unit::TestCase
     f.close
   end
 
+  def try_fdopen(fd, autoclose = true, level = 100)
+    if level > 0
+      try_fdopen(fd, autoclose, level - 1)
+      GC.start
+      level
+    else
+      IO.for_fd(fd, autoclose: autoclose)
+      nil
+    end
+  end
+
+  def test_autoclose
+    feature2250 = '[ruby-core:26222]'
+    pre = 'ft2250'
+
+    Tempfile.new(pre) do |t|
+      f = IO.for_fd(t.fileno)
+      assert_equal(true, f.autoclose?)
+      f.autoclose = false
+      assert_equal(false, f.autoclose?)
+      f.close
+      assert_nothing_raised(Errno::EBADF) {t.close}
+
+      t.open
+      f = IO.for_fd(t.fileno, autoclose: false)
+      assert_equal(false, f.autoclose?)
+      f.autoclose = true
+      assert_equal(true, f.autoclose?)
+      f.close
+      assert_raise(Errno::EBADF) {t.close}
+    end
+
+    Tempfile.new(pre) do |t|
+      try_fdopen(t.fileno)
+      assert_raise(Errno::EBADF) {t.close}
+    end
+
+    Tempfile.new(pre) do |t|
+      try_fdopen(f.fileno, false)
+      assert_nothing_raised(Errno::EBADF) {t.close}
+    end
+  end
+
   def test_open_redirect
     o = Object.new
     def o.to_open; self; end
@@ -1194,6 +1413,38 @@ class TestIO < Test::Unit::TestCase
         assert_equal("foo\n", f.gets)
       }
     end
+
+    open(__FILE__) do |f|
+      f.gets
+      f2 = open(t.path)
+      f2.gets
+      assert_nothing_raised {
+        f.reopen(f2)
+        assert_equal("bar\n", f.gets, '[ruby-core:24240]')
+      }
+    end
+
+    open(__FILE__) do |f|
+      f2 = open(t.path)
+      f.reopen(f2)
+      assert_equal("foo\n", f.gets)
+      assert_equal("bar\n", f.gets)
+      f.reopen(f2)
+      assert_equal("baz\n", f.gets, '[ruby-dev:39479]')
+    end
+  end
+
+  def test_reopen_inherit
+    mkcdtmpdir {
+      system(EnvUtil.rubybin, '-e', <<"End")
+        f = open("out", "w")
+        STDOUT.reopen(f)
+        STDERR.reopen(f)
+        system(#{EnvUtil.rubybin.dump}, '-e', 'STDOUT.print "out"')
+        system(#{EnvUtil.rubybin.dump}, '-e', 'STDERR.print "err"')
+End
+      assert_equal("outerr", File.read("out"))
+    }
   end
 
   def test_foreach
@@ -1257,6 +1508,24 @@ class TestIO < Test::Unit::TestCase
     assert_in_out_err(["-", t.path], "print while $<.gets", %w(foo bar baz), [])
   end
 
+  def test_print_separators
+    $, = ':'
+    $\ = "\n"
+    pipe(proc do |w|
+      w.print('a')
+      w.print('a','b','c')
+      w.close
+    end, proc do |r|
+      assert_equal("a\n", r.gets)
+      assert_equal("a:b:c\n", r.gets)
+      assert_nil r.gets
+      r.close
+    end)
+  ensure
+    $, = nil
+    $\ = nil
+  end
+
   def test_putc
     pipe(proc do |w|
       w.putc "A"
@@ -1299,22 +1568,30 @@ class TestIO < Test::Unit::TestCase
   end
 
   def test_initialize
+    return unless defined?(Fcntl::F_GETFL)
+
     t = make_tempfile
-    
+
     fd = IO.sysopen(t.path, "w")
     assert_kind_of(Integer, fd)
+    %w[r r+ w+ a+].each do |mode|
+      assert_raise(Errno::EINVAL, "#{mode} [ruby-dev:38571]") {IO.new(fd, mode)}
+    end
     f = IO.new(fd, "w")
     f.write("FOO\n")
     f.close
 
     assert_equal("FOO\n", File.read(t.path))
+  end
 
+  def test_reinitialize
+    t = make_tempfile
     f = open(t.path)
     assert_raise(RuntimeError) do
       f.instance_eval { initialize }
     end
   end
-  
+
   def test_new_with_block
     assert_in_out_err([], "r, w = IO.pipe; IO.new(r) {}", [], /^.+$/)
   end
@@ -1368,4 +1645,59 @@ class TestIO < Test::Unit::TestCase
       }
     }
   end
+
+  def test_tainted
+    t = make_tempfile
+    assert(File.read(t.path, 4).tainted?, '[ruby-dev:38826]')
+    assert(File.open(t.path) {|f| f.read(4)}.tainted?, '[ruby-dev:38826]')
+  end
+
+  def test_binmode_after_closed
+    t = make_tempfile
+    t.close
+    assert_raise(IOError) {t.binmode}
+  end
+
+  def test_threaded_flush
+    bug3585 = '[ruby-core:31348]'
+    src = %q{\
+      t = Thread.new { sleep 3 }
+      Thread.new {sleep 1; t.kill; p 'hi!'}
+      t.join
+    }.gsub(/^\s+/, '')
+    10.times.map do
+      Thread.start do
+        assert_in_out_err([], src, [%q["hi!"]])
+      end
+    end.each {|th| th.join}
+  end
+
+  def test_flush_in_finalizer1
+    require 'tempfile'
+    bug3910 = '[ruby-dev:42341]'
+    path = Tempfile.new("bug3910").path
+    fds = []
+    assert_nothing_raised(TypeError, bug3910) do
+      500.times {
+        f = File.open(path, "w")
+        fds << f.fileno
+        f.print "hoge"
+      }
+    end
+  ensure
+    fds.each {|fd| IO.for_fd(fd).close rescue next}
+  end
+
+  def test_flush_in_finalizer2
+    require 'tempfile'
+    bug3910 = '[ruby-dev:42341]'
+    path = Tempfile.new("bug3910").path
+    1.times do
+      io = open(path,"w")
+      io.print "hoge"
+    end
+    assert_nothing_raised(TypeError, bug3910) do
+      GC.start
+    end
+  end
 end
