@@ -346,6 +346,9 @@ class TestM17NComb < Test::Unit::TestCase
         assert_raise(Encoding::CompatibilityError) { s << s2 }
       end
     }
+
+    assert_equal("A\x84\x31\xA4\x39".force_encoding("GB18030"),
+                 "A".force_encoding("GB18030") << 0x8431A439)
   end
 
   def test_str_aref
@@ -709,7 +712,7 @@ class TestM17NComb < Test::Unit::TestCase
   def test_str_chomp
     combination(STRINGS, STRINGS) {|s1, s2|
       if !s1.ascii_only? && !s2.ascii_only? && !Encoding.compatible?(s1,s2)
-        if s1.bytesize > s2.bytesize 
+        if s1.bytesize > s2.bytesize
           assert_raise(Encoding::CompatibilityError) { s1.chomp(s2) }
         end
         next
@@ -727,15 +730,6 @@ class TestM17NComb < Test::Unit::TestCase
     STRINGS.each {|s|
       s = s.dup
       desc = "#{encdump s}.chop"
-      if !s.valid_encoding?
-        #assert_raise(ArgumentError, desc) { s.chop }
-        begin
-          s.chop
-        rescue ArgumentError
-          e = $!
-        end
-        next if e
-      end
       t = nil
       assert_nothing_raised(desc) { t = s.chop }
       assert(t.valid_encoding?) if s.valid_encoding?
@@ -1049,10 +1043,12 @@ class TestM17NComb < Test::Unit::TestCase
     STRINGS.each {|s|
       if /\0/ =~ a(s)
         assert_raise(ArgumentError) { s.intern }
-      else
+      elsif s.valid_encoding?
         sym = s.intern
         assert_equal(s, sym.to_s, "#{encdump s}.intern.to_s")
         assert_equal(sym, s.to_sym)
+      else
+        assert_raise(EncodingError) { s.intern }
       end
     }
   end
@@ -1066,7 +1062,7 @@ class TestM17NComb < Test::Unit::TestCase
   def test_str_oct
     STRINGS.each {|s|
       t = s.oct
-      t2 = a(s)[/\A[0-9a-fA-FxXbB]*/].oct
+      t2 = a(s)[/\A[0-9a-fA-FxX]*/].oct
       assert_equal(t2, t)
     }
   end
@@ -1102,7 +1098,7 @@ class TestM17NComb < Test::Unit::TestCase
         if s1.valid_encoding?
           assert_raise(Encoding::CompatibilityError) { s1.scan(s2) }
         else
-          assert_raise(ArgumentError, /invalid byte sequence/) { s1.scan(s2) }
+          assert_match(/invalid byte sequence/, assert_raise(ArgumentError) { s1.scan(s2) }.message)
         end
         next
       end
@@ -1354,7 +1350,7 @@ class TestM17NComb < Test::Unit::TestCase
     STRINGS.each {|s0|
       next if s0.empty?
       s = s0.dup
-      n = 1000
+      n = 300
       h = {}
       n.times {|i|
         if h[s]
