@@ -1,10 +1,7 @@
 require 'test/unit'
 require 'uri'
 
-module URI
-
-
-class TestGeneric < Test::Unit::TestCase
+class URI::TestGeneric < Test::Unit::TestCase
   def setup
     @url = 'http://a/b/c/d;p?q'
     @base_url = URI.parse(@url)
@@ -22,9 +19,9 @@ class TestGeneric < Test::Unit::TestCase
     assert_kind_of(URI::HTTP, @base_url)
 
     exp = [
-      'http', 
-      nil, 'a', URI::HTTP.default_port, 
-      '/b/c/d;p', 
+      'http',
+      nil, 'a', URI::HTTP.default_port,
+      '/b/c/d;p',
       'q',
       nil
     ]
@@ -36,8 +33,8 @@ class TestGeneric < Test::Unit::TestCase
     assert_kind_of(URI::FTP, url)
 
     exp = [
-      'ftp', 
-      nil, 'ftp.is.co.za', URI::FTP.default_port, 
+      'ftp',
+      nil, 'ftp.is.co.za', URI::FTP.default_port,
       'rfc/rfc1808.txt', nil,
     ]
     ary = uri_to_ary(url)
@@ -47,8 +44,8 @@ class TestGeneric < Test::Unit::TestCase
     assert_kind_of(URI::FTP, url)
 
     exp = [
-      'ftp', 
-      nil, 'ftp.is.co.za', URI::FTP.default_port, 
+      'ftp',
+      nil, 'ftp.is.co.za', URI::FTP.default_port,
       '/rfc/rfc1808.txt', nil,
     ]
     ary = uri_to_ary(url)
@@ -59,7 +56,7 @@ class TestGeneric < Test::Unit::TestCase
     assert_kind_of(URI::Generic, url)
 
     exp = [
-      'gopher', 
+      'gopher',
       nil, 'spinaltap.micro.umn.edu', nil, nil,
       '/00/Weather/California/Los%20Angeles', nil,
       nil,
@@ -73,9 +70,9 @@ class TestGeneric < Test::Unit::TestCase
     assert_kind_of(URI::HTTP, url)
 
     exp = [
-      'http', 
-      nil, 'www.math.uio.no', URI::HTTP.default_port, 
-      '/faq/compression-faq/part1.html', 
+      'http',
+      nil, 'www.math.uio.no', URI::HTTP.default_port,
+      '/faq/compression-faq/part1.html',
       nil,
       nil
     ]
@@ -87,7 +84,7 @@ class TestGeneric < Test::Unit::TestCase
     assert_kind_of(URI::Generic, url)
 
     exp = [
-      'mailto', 
+      'mailto',
       'mduerst@ifi.unizh.ch',
       []
     ]
@@ -99,8 +96,8 @@ class TestGeneric < Test::Unit::TestCase
     assert_kind_of(URI::Generic, url)
 
     exp = [
-      'news', 
-      nil, nil, nil, nil, 
+      'news',
+      nil, nil, nil, nil,
       nil, 'comp.infosystems.www.servers.unix',
       nil,
       nil
@@ -113,8 +110,8 @@ class TestGeneric < Test::Unit::TestCase
     assert_kind_of(URI::Generic, url)
 
     exp = [
-      'telnet', 
-      nil, 'melvyl.ucop.edu', nil, nil, 
+      'telnet',
+      nil, 'melvyl.ucop.edu', nil, nil,
       '/', nil,
       nil,
       nil
@@ -124,8 +121,8 @@ class TestGeneric < Test::Unit::TestCase
 
     # 7
     # reported by Mr. Kubota <em6t-kbt@asahi-net.or.jp>
-    assert_raises(URI::InvalidURIError) { URI.parse('http://a_b:80/') }
-    assert_raises(URI::InvalidURIError) { URI.parse('http://a_b/') }
+    assert_raise(URI::InvalidURIError) { URI.parse('http://a_b:80/') }
+    assert_raise(URI::InvalidURIError) { URI.parse('http://a_b/') }
 
     # 8
     # reported by m_seki
@@ -694,22 +691,19 @@ class TestGeneric < Test::Unit::TestCase
     assert_equal('http://foo:bar@zab:8080/?a=1#b123', uri.to_s)
 
     uri = URI.parse('http://example.com')
-    assert_raises(URI::InvalidURIError) { uri.password = 'bar' }
+    assert_raise(URI::InvalidURIError) { uri.password = 'bar' }
     uri.userinfo = 'foo:bar'
     assert_equal('http://foo:bar@example.com', uri.to_s)
-    assert_raises(URI::InvalidURIError) { uri.registry = 'bar' }
-    assert_raises(URI::InvalidURIError) { uri.opaque = 'bar' }
+    assert_raise(URI::InvalidURIError) { uri.registry = 'bar' }
+    assert_raise(URI::InvalidURIError) { uri.opaque = 'bar' }
 
     uri = URI.parse('mailto:foo@example.com')
-    assert_raises(URI::InvalidURIError) { uri.user = 'bar' }
-    assert_raises(URI::InvalidURIError) { uri.password = 'bar' }
-    assert_raises(URI::InvalidURIError) { uri.userinfo = ['bar', 'baz'] }
-    assert_raises(URI::InvalidURIError) { uri.host = 'bar' }
-    assert_raises(URI::InvalidURIError) { uri.port = 'bar' }
-    assert_raises(URI::InvalidURIError) { uri.path = 'bar' }
-    assert_raises(URI::InvalidURIError) { uri.query = 'bar' }
+    assert_raise(URI::InvalidURIError) { uri.user = 'bar' }
+    assert_raise(URI::InvalidURIError) { uri.password = 'bar' }
+    assert_raise(URI::InvalidURIError) { uri.userinfo = ['bar', 'baz'] }
+    assert_raise(URI::InvalidURIError) { uri.host = 'bar' }
+    assert_raise(URI::InvalidURIError) { uri.port = 'bar' }
+    assert_raise(URI::InvalidURIError) { uri.path = 'bar' }
+    assert_raise(URI::InvalidURIError) { uri.query = 'bar' }
   end
 end
-
-
-end
