@@ -1,6 +1,6 @@
 /**********************************************************************
 
-  vm_core.h - 
+  vm_core.h -
 
   $Author: yugui $
   created at: 04/01/01 19:41:38 JST
@@ -21,6 +21,7 @@
 #include "debug.h"
 #include "vm_opts.h"
 #include "id.h"
+#include "method.h"
 
 #if   defined(_WIN32)
 #include "thread_win32.h"
@@ -30,6 +31,21 @@
 #error "unsupported thread type"
 #endif
 
+#ifndef ENABLE_VM_OBJSPACE
+#ifdef _WIN32
+/*
+ * TODO: object space indenpendent st_table.
+ * socklist needs st_table in rb_w32_sysinit(), before object space
+ * initialization.
+ * It is too early now to change st_hash_type, since it breaks binary
+ * compatibility.
+ */
+#define ENABLE_VM_OBJSPACE 0
+#else
+#define ENABLE_VM_OBJSPACE 1
+#endif
+#endif
+
 #include <setjmp.h>
 #include <signal.h>
 
@@ -47,6 +63,10 @@
 #define va_init_list(a,b) va_start(a)
 #endif
 
+#if defined(SIGSEGV) && defined(HAVE_SIGALTSTACK) && defined(SA_SIGINFO) && !defined(__NetBSD__)
+#define USE_SIGALTSTACK
+#endif
+
 /*****************/
 /* configuration */
 /*****************/
@@ -90,8 +110,16 @@
 #define UNLIKELY(x) (x)
 #endif /* __GNUC__ >= 3 */
 
+#if __GNUC__ >= 3
+#define UNINITIALIZED_VAR(x) x = x
+#else
+#define UNINITIALIZED_VAR(x) x
+#endif
+
 typedef unsigned long rb_num_t;
 
+/* iseq data type */
+
 struct iseq_compile_data_ensure_node_stack;
 
 typedef struct rb_compile_option_struct {
@@ -106,6 +134,16 @@ typedef struct rb_compile_option_struct {
     int debug_level;
 } rb_compile_option_t;
 
+struct iseq_inline_cache_entry {
+    VALUE ic_vmstat;
+    VALUE ic_class;
+    union {
+	VALUE value;
+	rb_method_entry_t *method;
+	long index;
+    } ic_value;
+};
+
 #if 1
 #define GetCoreDataFromValue(obj, type, ptr) do { \
     ptr = (type*)DATA_PTR(obj); \
@@ -127,6 +165,7 @@ struct rb_iseq_struct {
     VALUE type;          /* instruction sequence type */
     VALUE name;	         /* String: iseq name */
     VALUE filename;      /* file information where this sequence from */
+    VALUE filepath;      /* real file path or nil */
     VALUE *iseq;         /* iseq (insn number and openrads) */
     VALUE *iseq_encoded; /* encoded iseq */
     unsigned long iseq_size;
@@ -136,13 +175,16 @@ struct rb_iseq_struct {
 
     /* insn info, must be freed */
     struct iseq_insn_info_entry *insn_info_table;
-    unsigned long insn_info_size;
+    size_t insn_info_size;
 
     ID *local_table;		/* must free */
     int local_table_size;
 
     /* method, class frame: sizeof(vars) + 1, block frame: sizeof(vars) */
-    int local_size; 
+    int local_size;
+
+    struct iseq_inline_cache_entry *ic_entries;
+    int ic_size;
 
     /**
      * argument information
@@ -177,7 +219,7 @@ struct rb_iseq_struct {
     int arg_size;
     VALUE *arg_opt_table;
 
-    int stack_max; /* for stack overflow check */
+    size_t stack_max; /* for stack overflow check */
 
     /* catch table */
     struct iseq_catch_table_entry *catch_table;
@@ -195,7 +237,7 @@ struct rb_iseq_struct {
     VALUE orig;			/* non-NULL if its data have origin */
 
     /* block inlining */
-    /* 
+    /*
      * NODE *node;
      * void *special_block_builder;
      * void *cached_special_block_builder;
@@ -220,11 +262,14 @@ enum ruby_special_exceptions {
     ruby_special_error_count
 };
 
-typedef struct rb_iseq_struct rb_iseq_t;
-
 #define GetVMPtr(obj, ptr) \
   GetCoreDataFromValue(obj, rb_vm_t, ptr)
 
+#if defined(ENABLE_VM_OBJSPACE) && ENABLE_VM_OBJSPACE
+struct rb_objspace;
+void rb_objspace_free(struct rb_objspace *);
+#endif
+
 typedef struct rb_vm_struct {
     VALUE self;
 
@@ -251,7 +296,7 @@ typedef struct rb_vm_struct {
     VALUE load_path;
     VALUE loaded_features;
     struct st_table *loading_table;
-    
+
     /* signal */
     struct {
 	VALUE cmd;
@@ -266,6 +311,8 @@ typedef struct rb_vm_struct {
     VALUE verbose, debug, progname;
     VALUE coverages;
 
+    struct unlinked_method_entry_list_entry *unlinked_method_entry_list;
+
 #if defined(ENABLE_VM_OBJSPACE) && ENABLE_VM_OBJSPACE
     struct rb_objspace *objspace;
 #endif
@@ -282,8 +329,7 @@ typedef struct {
     VALUE *dfp;			/* cfp[7] / block[2] */
     rb_iseq_t *block_iseq;	/* cfp[8] / block[3] */
     VALUE proc;			/* cfp[9] / block[4] */
-    ID method_id;               /* cfp[10] saved in special case */
-    VALUE method_class;         /* cfp[11] saved in special case */
+    const rb_method_entry_t *me;/* cfp[10] */
 } rb_control_frame_t;
 
 typedef struct rb_block_struct {
@@ -314,8 +360,8 @@ struct rb_vm_tag {
     struct rb_vm_tag *prev;
 };
 
-struct rb_vm_trap_tag {
-    struct rb_vm_trap_tag *prev;
+struct rb_vm_protect_tag {
+    struct rb_vm_protect_tag *prev;
 };
 
 #define RUBY_VM_VALUE_CACHE_SIZE 0x1000
@@ -340,12 +386,15 @@ typedef struct rb_thread_struct
     int safe_level;
     int raised_flag;
     VALUE last_status; /* $? */
-    
+
     /* passing state */
     int state;
 
     /* for rb_iterate */
-    rb_block_t *passed_block;
+    const rb_block_t *passed_block;
+
+    /* for bmethod */
+    const rb_method_entry_t *passed_me;
 
     /* for load(true) */
     VALUE top_self;
@@ -378,10 +427,10 @@ typedef struct rb_thread_struct
     struct rb_unblock_callback unblock;
     VALUE locking_mutex;
     struct rb_mutex_struct *keeping_mutexes;
-    int transition_for_lock;
+    volatile int transition_for_lock;
 
     struct rb_vm_tag *tag;
-    struct rb_vm_trap_tag *trap_tag;
+    struct rb_vm_protect_tag *protect_tag;
 
     int parse_in_eval;
     int mild_compile_error;
@@ -428,17 +477,20 @@ typedef struct rb_thread_struct
     /* misc */
     int method_missing_reason;
     int abort_on_exception;
+#ifdef USE_SIGALTSTACK
+    void *altstack;
+#endif
 } rb_thread_t;
 
 /* iseq.c */
-VALUE rb_iseq_new(NODE*, VALUE, VALUE, VALUE, VALUE);
-VALUE rb_iseq_new_top(NODE *node, VALUE name, VALUE filename, VALUE parent);
-VALUE rb_iseq_new_main(NODE *node, VALUE filename);
-VALUE rb_iseq_new_with_bopt(NODE*, VALUE, VALUE, VALUE, VALUE, VALUE, VALUE);
-VALUE rb_iseq_new_with_opt(NODE*, VALUE, VALUE, VALUE, VALUE, VALUE, const rb_compile_option_t*);
+VALUE rb_iseq_new(NODE*, VALUE, VALUE, VALUE, VALUE, VALUE);
+VALUE rb_iseq_new_top(NODE *node, VALUE name, VALUE filename, VALUE filepath, VALUE parent);
+VALUE rb_iseq_new_main(NODE *node, VALUE filename, VALUE filepath);
+VALUE rb_iseq_new_with_bopt(NODE*, VALUE, VALUE, VALUE, VALUE, VALUE, VALUE, VALUE);
+VALUE rb_iseq_new_with_opt(NODE*, VALUE, VALUE, VALUE, VALUE, VALUE, VALUE, const rb_compile_option_t*);
 VALUE rb_iseq_compile(VALUE src, VALUE file, VALUE line);
 VALUE rb_iseq_disasm(VALUE self);
-VALUE rb_iseq_disasm_insn(VALUE str, VALUE *iseqval, int pos, rb_iseq_t *iseq, VALUE child);
+int rb_iseq_disasm_insn(VALUE str, VALUE *iseqval, size_t pos, rb_iseq_t *iseq, VALUE child);
 const char *ruby_node_name(int node);
 int rb_iseq_first_lineno(rb_iseq_t *iseq);
 
@@ -450,11 +502,6 @@ RUBY_EXTERN VALUE rb_mRubyVMFrozenCore;
 /* each thread has this size stack : 128KB */
 #define RUBY_VM_THREAD_STACK_SIZE (128 * 1024)
 
-struct global_entry {
-    struct global_variable *var;
-    ID id;
-};
-
 #define GetProcPtr(obj, ptr) \
   GetCoreDataFromValue(obj, rb_proc_t, ptr)
 
@@ -484,9 +531,10 @@ typedef struct {
 
 typedef struct {
     VALUE env;
+    VALUE filename;
+    unsigned short line_no;
 } rb_binding_t;
 
-
 /* used by compile time and send insn */
 #define VM_CALL_ARGS_SPLAT_BIT     (0x01 << 1)
 #define VM_CALL_ARGS_BLOCKARG_BIT  (0x01 << 2)
@@ -495,10 +543,11 @@ typedef struct {
 #define VM_CALL_TAILCALL_BIT       (0x01 << 5)
 #define VM_CALL_TAILRECURSION_BIT  (0x01 << 6)
 #define VM_CALL_SUPER_BIT          (0x01 << 7)
-#define VM_CALL_SEND_BIT           (0x01 << 8)
+#define VM_CALL_OPT_SEND_BIT       (0x01 << 8)
 
-#define VM_SPECIAL_OBJECT_VMCORE   0x01
-#define VM_SPECIAL_OBJECT_CBASE    0x02
+#define VM_SPECIAL_OBJECT_VMCORE       0x01
+#define VM_SPECIAL_OBJECT_CBASE        0x02
+#define VM_SPECIAL_OBJECT_CONST_BASE  0x03
 
 #define VM_FRAME_MAGIC_METHOD 0x11
 #define VM_FRAME_MAGIC_BLOCK  0x21
@@ -518,19 +567,17 @@ typedef struct {
 /* other frame flag */
 #define VM_FRAME_FLAG_PASSED 0x0100
 
-
 #define RUBYVM_CFUNC_FRAME_P(cfp) \
   (VM_FRAME_TYPE(cfp) == VM_FRAME_MAGIC_CFUNC)
 
+/* inline cache */
+typedef struct iseq_inline_cache_entry *IC;
 
-/* inline (method|const) cache */
-#define NEW_INLINE_CACHE_ENTRY() NEW_WHILE(Qundef, 0, 0)
-#define ic_class  u1.value
-#define ic_method u2.node
-#define ic_value  u2.value
-#define ic_vmstat u3.cnt
-typedef NODE *IC;
+extern VALUE ruby_vm_global_state_version;
 
+#define GET_VM_STATE_VERSION() (ruby_vm_global_state_version)
+#define INC_VM_STATE_VERSION() \
+  (ruby_vm_global_state_version = (ruby_vm_global_state_version+1) & 0x8fffffff)
 void rb_vm_change_state(void);
 
 typedef VALUE CDHASH;
@@ -559,8 +606,6 @@ typedef rb_control_frame_t *
 #define RUBY_VM_NORMAL_ISEQ_P(ptr) \
   (ptr && !RUBY_VM_IFUNC_P(ptr))
 
-#define RUBY_VM_CLASS_SPECIAL_P(ptr) (((VALUE)(ptr)) & 0x02)
-
 #define RUBY_VM_GET_BLOCK_PTR_IN_CFP(cfp) ((rb_block_t *)(&(cfp)->self))
 #define RUBY_VM_GET_CFP_FROM_BLOCK_PTR(b) \
   ((rb_control_frame_t *)((VALUE *)(b) - 5))
@@ -575,9 +620,7 @@ extern void rb_vmdebug_stack_dump_raw(rb_thread_t *, rb_control_frame_t *);
 #define SDR2(cfp) rb_vmdebug_stack_dump_raw(GET_THREAD(), (cfp))
 void rb_vm_bugreport(void);
 
-
 /* functions about thread/vm execution */
-
 VALUE rb_iseq_eval(VALUE iseqval);
 VALUE rb_iseq_eval_main(VALUE iseqval);
 void rb_enable_interrupt(void);
@@ -585,18 +628,29 @@ void rb_disable_interrupt(void);
 int rb_thread_method_id_and_class(rb_thread_t *th, ID *idp, VALUE *klassp);
 
 VALUE rb_vm_invoke_proc(rb_thread_t *th, rb_proc_t *proc, VALUE self,
-			int argc, const VALUE *argv, rb_block_t *blockptr);
+			int argc, const VALUE *argv, const rb_block_t *blockptr);
 VALUE rb_vm_make_proc(rb_thread_t *th, const rb_block_t *block, VALUE klass);
 VALUE rb_vm_make_env_object(rb_thread_t *th, rb_control_frame_t *cfp);
 
+void rb_thread_start_timer_thread(void);
+void rb_thread_stop_timer_thread(void);
+void rb_thread_reset_timer_thread(void);
 void *rb_thread_call_with_gvl(void *(*func)(void *), void *data1);
 int ruby_thread_has_gvl_p(void);
+VALUE rb_make_backtrace(void);
+typedef int rb_backtrace_iter_func(void *, VALUE, int, VALUE);
+int rb_backtrace_each(rb_backtrace_iter_func *iter, void *arg);
 rb_control_frame_t *rb_vm_get_ruby_level_next_cfp(rb_thread_t *th, rb_control_frame_t *cfp);
+int rb_vm_get_sourceline(const rb_control_frame_t *);
+VALUE rb_name_err_mesg_new(VALUE obj, VALUE mesg, VALUE recv, VALUE method);
 
 NOINLINE(void rb_gc_save_machine_context(rb_thread_t *));
 
 #define sysstack_error GET_VM()->special_exceptions[ruby_error_sysstack]
 
+VALUE rb_str_resurrect(VALUE str);
+VALUE rb_ary_resurrect(VALUE ary);
+
 /* for thread */
 
 #if RUBY_VM_THREAD_MODEL == 2
@@ -620,11 +674,17 @@ extern rb_vm_t *ruby_current_vm;
 #define RUBY_VM_SET_FINALIZER_INTERRUPT(th) ((th)->interrupt_flag |= 0x04)
 #define RUBY_VM_INTERRUPTED(th) ((th)->interrupt_flag & 0x02)
 
-void rb_thread_execute_interrupts(rb_thread_t *);
+void rb_threadptr_check_signal(rb_thread_t *mth);
+void rb_threadptr_signal_raise(rb_thread_t *th, int sig);
+void rb_threadptr_signal_exit(rb_thread_t *th);
+void rb_threadptr_execute_interrupts(rb_thread_t *);
+
+void rb_thread_lock_unlock(rb_thread_lock_t *);
+void rb_thread_lock_destroy(rb_thread_lock_t *);
 
 #define RUBY_VM_CHECK_INTS_TH(th) do { \
   if (UNLIKELY(th->interrupt_flag)) { \
-    rb_thread_execute_interrupts(th); \
+    rb_threadptr_execute_interrupts(th); \
   } \
 } while (0)
 
